from collections.abc import Mapping
from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="CreateModelBody")


@_attrs_define
class CreateModelBody:
    """
    Attributes:
        project_id (Union[Unset, str]): Project ID to attach this model.
        payload (Union[None, Unset, str]): The serialized model data.
    """

    project_id: Union[Unset, str] = UNSET
    payload: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        project_id = self.project_id

        payload: Union[None, Unset, str]
        if isinstance(self.payload, Unset):
            payload = UNSET
        else:
            payload = self.payload

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if project_id is not UNSET:
            field_dict["project_id"] = project_id
        if payload is not UNSET:
            field_dict["payload"] = payload

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        project_id = d.pop("project_id", UNSET)

        def _parse_payload(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        payload = _parse_payload(d.pop("payload", UNSET))

        create_model_body = cls(
            project_id=project_id,
            payload=payload,
        )

        create_model_body.additional_properties = d
        return create_model_body

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.scaleway_qaas_v1_alpha_1_model import ScalewayQaasV1Alpha1Model


T = TypeVar("T", bound="ScalewayQaasV1Alpha1ListModelsResponse")


@_attrs_define
class ScalewayQaasV1Alpha1ListModelsResponse:
    """
    Attributes:
        total_count (Union[Unset, int]): Total number of models.
        models (Union[Unset, list['ScalewayQaasV1Alpha1Model']]): List of models.
    """

    total_count: Union[Unset, int] = UNSET
    models: Union[Unset, list["ScalewayQaasV1Alpha1Model"]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        total_count = self.total_count

        models: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.models, Unset):
            models = []
            for models_item_data in self.models:
                models_item = models_item_data.to_dict()
                models.append(models_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if total_count is not UNSET:
            field_dict["total_count"] = total_count
        if models is not UNSET:
            field_dict["models"] = models

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.scaleway_qaas_v1_alpha_1_model import ScalewayQaasV1Alpha1Model

        d = dict(src_dict)
        total_count = d.pop("total_count", UNSET)

        models = []
        _models = d.pop("models", UNSET)
        for models_item_data in _models or []:
            models_item = ScalewayQaasV1Alpha1Model.from_dict(models_item_data)

            models.append(models_item)

        scaleway_qaas_v1_alpha_1_list_models_response = cls(
            total_count=total_count,
            models=models,
        )

        scaleway_qaas_v1_alpha_1_list_models_response.additional_properties = d
        return scaleway_qaas_v1_alpha_1_list_models_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.scaleway_qaas_v1_alpha_1_process_result import (
        ScalewayQaasV1Alpha1ProcessResult,
    )


T = TypeVar("T", bound="ScalewayQaasV1Alpha1ListProcessResultsResponse")


@_attrs_define
class ScalewayQaasV1Alpha1ListProcessResultsResponse:
    """
    Attributes:
        total_count (Union[Unset, int]): Total number of results.
        process_results (Union[Unset, list['ScalewayQaasV1Alpha1ProcessResult']]): List of results.
    """

    total_count: Union[Unset, int] = UNSET
    process_results: Union[Unset, list["ScalewayQaasV1Alpha1ProcessResult"]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        total_count = self.total_count

        process_results: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.process_results, Unset):
            process_results = []
            for process_results_item_data in self.process_results:
                process_results_item = process_results_item_data.to_dict()
                process_results.append(process_results_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if total_count is not UNSET:
            field_dict["total_count"] = total_count
        if process_results is not UNSET:
            field_dict["process_results"] = process_results

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.scaleway_qaas_v1_alpha_1_process_result import (
            ScalewayQaasV1Alpha1ProcessResult,
        )

        d = dict(src_dict)
        total_count = d.pop("total_count", UNSET)

        process_results = []
        _process_results = d.pop("process_results", UNSET)
        for process_results_item_data in _process_results or []:
            process_results_item = ScalewayQaasV1Alpha1ProcessResult.from_dict(
                process_results_item_data
            )

            process_results.append(process_results_item)

        scaleway_qaas_v1_alpha_1_list_process_results_response = cls(
            total_count=total_count,
            process_results=process_results,
        )

        scaleway_qaas_v1_alpha_1_list_process_results_response.additional_properties = d
        return scaleway_qaas_v1_alpha_1_list_process_results_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# Copyright 2025 Scaleway
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os
import time
import uuid

from scaleway_qaas_client.v1alpha1 import QaaSClient

_RANDOM_UUID = str(uuid.uuid4())
_TEST_PLATFORM_NAME = os.environ.get("TEST_PLATFORM_NAME", "aer_simulation_pop_c16m128")
_TEST_APPLICATION_NAME = os.environ.get("TEST_APPLICATION_NAME", "H2 VQE")


def _get_client() -> QaaSClient:
    client = QaaSClient(
        project_id=os.environ["SCALEWAY_PROJECT_ID"],
        secret_key=os.environ["SCALEWAY_SECRET_KEY"],
        url=os.environ["SCALEWAY_API_URL"],
    )

    return client


def test_list_platform():
    client = _get_client()

    platforms = client.list_platforms()

    assert platforms is not None
    assert len(platforms) > 0


def test_list_platforms_by_backend():
    client = _get_client()

    platforms = client.list_platforms(backend_name="aer")

    assert len(platforms) > 0

    for platform in platforms:
        assert platform.backend_name == "aer"


def test_list_platforms_by_provider():
    client = _get_client()

    platforms = client.list_platforms(provider_name="quandela")

    assert len(platforms) > 0

    for platform in platforms:
        assert platform.provider_name == "quandela"


def test_list_platforms_by_unexisting_provider():
    client = _get_client()

    platforms = client.list_platforms(provider_name=_RANDOM_UUID)

    assert platforms == []


def test_list_platforms_by_unexisting_backend():
    client = _get_client()

    platforms = client.list_platforms(backend_name=_RANDOM_UUID)

    assert platforms == []


def test_list_platforms_by_name():
    client = _get_client()

    platforms = client.list_platforms(name=_TEST_PLATFORM_NAME)

    for platform in platforms:
        assert platform.name == _TEST_PLATFORM_NAME

    assert len(platforms) > 0


def test_list_platforms_by_unexisting_name():
    client = _get_client()

    platforms = client.list_platforms(name=_RANDOM_UUID)

    assert platforms == []


def test_list_sessions_unexisting_platform():
    client = _get_client()

    sessions = client.list_sessions(platform_id=_RANDOM_UUID)

    assert sessions == []


def test_create_delete_session():
    client = _get_client()

    try:
        platforms = client.list_platforms(name=_TEST_PLATFORM_NAME)

        assert platforms is not None
        assert len(platforms) == 1

        target_platform = platforms[0]

        assert target_platform.id is not None

        max_duration = "2m"
        max_idle_duration = "2m"

        session = client.create_session(
            platform_id=target_platform.id,
            max_duration=max_duration,
            max_idle_duration=max_idle_duration,
        )

        assert session is not None
        assert session.id is not None
        assert session.platform_id == target_platform.id

        while session.status == "starting":
            session = client.get_session(session.id)
            time.sleep(3)

        assert session.status == "running"

        session_acls = client.list_session_acls(session.id)
        assert len(session_acls) > 0

        session = client.terminate_session(session.id)

        while session.status == "stopping":
            session = client.get_session(session.id)
            time.sleep(3)

        assert session.status == "stopped"
    finally:
        client.delete_session(session.id)


def test_create_session_same_deduplication_id():
    client = _get_client()

    try:
        platforms = client.list_platforms(name=_TEST_PLATFORM_NAME)

        assert len(platforms) > 0

        platform = platforms[0]

        max_duration = "2m"
        max_idle_duration = "2m"
        deduplication_id = f"hihaaa-{_RANDOM_UUID}"

        session = client.create_session(
            platform_id=platform.id,
            max_duration=max_duration,
            max_idle_duration=max_idle_duration,
            deduplication_id=deduplication_id,
        )

        assert session is not None
        assert session.id is not None
        assert session.platform_id == platform.id
        assert session.deduplication_id == deduplication_id
        assert session.status in ["starting", "running"]

        second_session = client.create_session(
            platform_id=platform.id,
            deduplication_id=deduplication_id,
        )

        assert second_session != None
        assert second_session.id == session.id
        assert second_session.project_id == session.project_id
        assert second_session.platform_id == session.platform_id
        assert second_session.name == session.name
        assert second_session.max_duration == session.max_duration
        assert second_session.max_idle_duration == session.max_idle_duration
        assert second_session.status in session.status
        assert second_session.deduplication_id == session.deduplication_id
    finally:
        client.delete_session(session.id)


def test_list_applications():
    client = _get_client()

    applications = client.list_applications()

    assert len(applications) > 0


def test_list_applications_by_name():
    client = _get_client()

    applications = client.list_applications(name=_TEST_APPLICATION_NAME)

    assert len(applications) > 0

    for application in applications:
        assert application.name == _TEST_APPLICATION_NAME


def test_run_process():
    client = _get_client()

    process_inputs = {
        "Custom VQE": '{ "max_iterations": 1, "hamiltonian_strings" : ["XIIX", "ZZYY", "ZXYY", "ZZZZ"], "hamiltonian_weights" : [ -0.5, 1, 2.44, 5 ] }',
        "CVar VQE": '{ "max_iterations": 3, "qubo_matrix" : [ [ 31, -500 ], [ -500, 32 ] ] }',
        "Chemistry VQE": '{"max_iterations": 1, "geometry": [ {"coordinates": [ 0, 0, 0 ], "element": "Li" }, { "coordinates": [ 0, 0, 0.7414 ], "element": "H" }]}',
        "H2 VQE": '{"max_iterations": 2, "geometry": [ {"coordinates": [ 0, 0, 0 ], "element": "H" }, { "coordinates": [ 0, 0, 0.7414 ], "element": "H" }]}',
        "Graph Isomorphism": '{ "graph_a" : [ [ 0, 1 ], [ 1, 2 ], [2, 3], [2, 4], [3, 4] ], "graph_b" : [ [ 0, 1 ], [ 1, 2 ], [2, 3], [2, 4], [3, 4] ], "epsilon" : 10, "error" : 0.1, "algo" : "Laplacian PP", "max_iterations" : 3, "nb_samples" : 1000, "nb_samples_min_accepted" : 10}',
        "Graph DSI": '{ "graph" : [ [ 0, 1 ], [ 1, 2 ], [2, 3], [2, 4], [3, 4] ], "max_iterations" : 10, "nb_samples" : 10000, "size_subgraph" : 3, "seed" : [ 0 ], "nb_samples_min_accepted" : 1000}',
    }

    platforms = client.list_platforms(name=_TEST_PLATFORM_NAME)

    assert len(platforms) > 0

    platform = platforms[0]

    if platform.provider_name != "quandela":
        print("SKIP RUN PROCESS : ONLY QUANDELA PLATFORMS CAN RUN PROCESS")
        return

    assert _TEST_APPLICATION_NAME in process_inputs

    applications = client.list_applications(name=_TEST_APPLICATION_NAME)

    assert len(applications) > 0

    application = applications[0]

    process = client.create_process(
        platform_id=platform.id,
        application_id=application.id,
        input=process_inputs[_TEST_APPLICATION_NAME],
    )

    assert process.platform_id == platform.id
    assert process.application_id == application.id
    assert process.status == "starting"

    assert process.input_ == process_inputs[_TEST_APPLICATION_NAME]
    assert process.progress_message == ""
    assert process.tags == []

    while True:
        time.sleep(3)
        process = client.get_process(process.id)
        assert process.status in [
            "starting",
            "running",
            "completed",
        ]
        print(process.progress, process.progress_message)
        if process.status == "completed":
            results = client.list_process_results(process.id)
            assert len(results) > 0
            break
